<?php
    /** CODE FOR DISPLAYING RATE HTML **/
    include( plugin_dir_path( __FILE__ ) . 'pm-rates-helper.php');

    class RateTable {
        /**
        * __construct()
        */
        public function __construct() {
            add_action( 'init', [ $this, 'register' ] );
            add_action('enqueue_block_editor_assets', [ $this, 'enqueue_block_assets']);
        }


        /**
        * Register Assets()
        *
        * @return void
        */
        public function enqueue_block_assets(): void {
            wp_enqueue_script(
                'pmrates/blocks-js',
                plugins_url(null, __FILE__).'/pm-rates-block.js',
                ['wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor'],
                null
            );

            wp_enqueue_style(
                'pmrates/blocks-editor-css',
                plugins_url(null, __FILE__).'/pm-rates-block.css',
                [],
                null
            );
        }

        /**
        * register()
        *
        * @return void
        */

        public function register(): void {
            register_block_type( 'pmrates/rate-dynamic', array(
                "apiVersion" => 2,
                "name" => "pmrates/rate-dynamic",
                "title" => "Pannos: Basic Rate Table",
                "category" => "text",
                "description" => "Display a Basic Rate Table.",
                "attributes" => [
                    "rateTableID" => [
                        "type" => "string",
                        "default" => ""
                    ],
                    "showIcon" => [
                        "type" => "string",
                        "default" => "no"
                    ],
                    "rateTableAnchor" => [
                        "type" => "string",
                        "default" => ""
                    ],
                    "rateTableMargin" => [
                        "type" => "string",
                        "default" => "margin-bottom-large"
                    ]
                ],
                "supports"=> [
                    "customClassName" => false
                ],
                'render_callback' => [ $this, 'render' ]
            ) );
        }

        /**
        * render()
        *
        * @param array $attributes Block attributes
        * @return string
        */
        public function render( $attributes ): string {
            if(isset($attributes['rateTableID']) && $attributes['rateTableID'] != null && !empty($attributes['rateTableID'])) {
                $debug = print_r($attributes, true);
                $post = get_post($attributes['rateTableID']);
                if($post) {
                    $rate_table_icon = "";
                    if(has_post_thumbnail($post) && !empty(get_the_post_thumbnail_url($post)) && $attributes['showIcon'] === 'yes') {
                        $rate_table_icon = '<img src="'.get_the_post_thumbnail_url($post).'" width="100" height="100" alt="" /><br/>';
                    }
                    $formatted_table_title = '';
                    if(!empty($post->table_title) && isset($attributes['showtitle']) && $attributes['showtitle'] == 1) {
                        $formatted_table_title = '<h3 class="rate-table-title">'.$rate_table_icon.$post->table_title.'</h3>';
                    }                    
                    if(!empty($post->table_title) && isset($attributes['showprint']) && $attributes['showprint'] == 1) {
                        $formatted_table_title = '<button class="rates-print-btn js-print-rate">Print<span class="visuallyhidden"> '.!empty($post->table_title).'</span></button>';
                    }
                    $formatted_table_subtitle = '';
                    if(!empty($post->table_subtitle)) {
                        $formatted_table_subtitle = '<p class="subtitle">'.$post->table_subtitle.'</p>';
                    }
                    $formatted_table_intro = empty($post->table_intro) ? '' : '<div class="body">'.$post->table_intro.'</div>';
                    if(isset($attributes['showdate']) && $attributes['showdate'] == 1){

                        if( isset($attributes['dateformat']) && (!empty($attributes['dateformat']) ) ){
                            $date_format = $attributes['dateformat'];
                        } else {
                            $date_format = "l, F j, Y";
                        }
                        $formatted_effective_date = date($date_format);
                        if(isset($post->effective_date) && !empty($post->effective_date)) {
                            $effectiveDate = DateTime::createFromFormat('Y-m-d', $post->effective_date);
                            $formatted_effective_date = $effectiveDate->format($date_format);
                        }

                        $formatted_effective_date_text = $post->show_effective_date == "false" ? '' : '<p class="effective_date"><strong>Effective Date:</strong> '.$formatted_effective_date.'</p>';
                    } else {
                        $formatted_effective_date_text = '';
                    }
                    $hide_columns = array();
                    $hide_rows = array();
                    if(empty($attributes['showAll'])){

                        if( isset($attributes['hidecolumns']) && (!empty($attributes['hidecolumns']) || is_numeric($attributes['hidecolumns'])) ){
                            $hide_columns_items = explode(',', $attributes['hidecolumns']);
                            foreach ($hide_columns_items as $hc_count => $hide_columns_item) {
                                $range = explode('-', $hide_columns_item);
                                if(count($range) > 1){
                                    foreach (range($range[0], $range[1]) as $number) {
                                        $hide_columns[$number] = true;
                                    }
                                } else {
                                    $hide_columns[trim($range[0])] = true;
                                }
                            }
                        }
                        if(isset($attributes['hiderows']) && (!empty($attributes['hiderows']) || is_numeric($attributes['hiderows'])) ){
                            $hide_rows_items = explode(',', $attributes['hiderows']);
                            foreach ($hide_rows_items as $hc_count => $hide_rows_item) {
                                $range = explode('-', $hide_rows_item);
                                if(count($range) > 1){
                                    foreach (range($range[0], $range[1]) as $number) {
                                        $hide_rows[$number] = true;
                                    }
                                } else {
                                    $hide_rows[trim($range[0])] = true;
                                }
                            }
                            
                        }

                    }
                    $debug_hide = '<!-- hide columns: '.print_r($hide_columns,true).' hide rows: '.print_r($hide_rows,true).' -->';
                    $formatted_table_html = showRateTableMarkup($post->table_data, $post->table_caption, $hide_columns, $hide_rows, $attributes['rateTableID']);
                    $formatted_table_disclosure = (!empty($post->table_disclosure) && isset($attributes['showdisclosure']) && $attributes['showdisclosure'] == 1) ? '<div class="rate_disclosure">'.$post->table_disclosure.'</div>' : '';
                    $formatted_ctas = '';
                    $cta_json = array();
                    if(null !== $post->cta_data && !empty($post->cta_data)) {
                        $cta_json = preg_split("/::ctadivide::/", html_entity_decode($post->cta_data, ENT_QUOTES));
                        if(!empty($cta_json[0]['text'])) {
                            $formatted_ctas .= '<ul class="actions">';
                            foreach($cta_json as $i => $cta) {
                                $decoded_cta = json_decode($cta, true);
                                $formatted_ctas .= '<li><a class="cta-lnk-black ';

                                $formatted_ctas .= '" target="';
                                if(isset($decoded_cta['target']) && !empty($decoded_cta['target'])) {
                                    $formatted_ctas .= $decoded_cta['target'];
                                }
                                $formatted_ctas .= '" href="';
                                if(isset($decoded_cta['custom_url']) && !empty($decoded_cta['custom_url'])) {
                                    $formatted_ctas .= $decoded_cta['custom_url'];
                                } else if(isset($decoded_cta['url']) && !empty($decoded_cta['url'])) {
                                    $formatted_ctas .= get_page_link( $decoded_cta['url'] );
                                }
                                if(isset($decoded_cta['anchor']) && !empty($decoded_cta['anchor'])) {
                                    $outputHTML .= $decoded_cta['anchor'];
                                }
                                $formatted_ctas .= '">';
                                if(isset($decoded_cta['text']) && !empty($decoded_cta['text'])) {
                                    $formatted_ctas .= $decoded_cta['text'];
                                }
                                if(isset($decoded_cta['ada_text']) && !empty($decoded_cta['ada_text'])) {
                                    $formatted_ctas .= '<span class="visuallyhidden">&nbsp;'.$decoded_cta['ada_text'].'</span>';
                                }
                                $formatted_ctas .= '</a></li>';
                            }
                            $formatted_ctas .= '</ul>';
                        }
                    }

                    return <<<HTML
                        <section id="{$attributes['rateTableAnchor']}" class="mod-rate-table {$attributes['rateTableMargin']}"><div class="hidden">$debug</div>$debug_hide
                            <div class="content rate_table">
                                <div class="intro">
                                    $formatted_table_title
                                    $formatted_table_subtitle
                                    $formatted_table_intro
                                    <div class="body">
                                        $formatted_effective_date_text
                                    </div>
                                </div>
                                <div class="table-wrapper">$formatted_table_html</div>
                                $formatted_ctas
                                $formatted_table_disclosure
                            </div>
                        </section>
                    HTML;
                } else {
                    return '';
                }
            } else {
                return '';
            }
        }
    }
    new RateTable;
?>
