<?php
    // error_reporting(E_ALL);
    // ini_set('display_errors', 1);
    /*
        Plugin Name: Pannos Rates WP
        Plugin URI: https://www.pannosmarketing.com/
        Description: For managing Rate Tables imported from previous site and creating new ones.
        Version: 1.3.5
        Author: Matt Larrabee & Max Hann
        Author URI: https://www.pannosmarketing.com/
        License: GPLv2 or later
        Text Domain: pmrates
    */

    function pmrates_rate_scripts() {
        wp_enqueue_style(
            'pmrates_css',
            plugin_dir_url( __FILE__ ).'styles/pm-rates.css',
            [ 'wp-edit-blocks' ],
            filemtime( plugin_dir_path( __FILE__ ).'styles/pm-rates.css' )
        );
        wp_enqueue_script( 'pmrates_js', plugin_dir_url( __FILE__ ).'scripts/pm-rates.js', '', '', true );
        wp_enqueue_editor();
    }
    add_action( 'admin_enqueue_scripts', 'pmrates_rate_scripts' );


    function pmrates_register_post_type() {
        // movies
        $labels = array(
            'name' => __( 'Rates' , 'pmrates' ),
            'singular_name' => __( 'Rate' , 'pmrates' ),
            'add_new' => __( 'New Rate Table' , 'pmrates' ),
            'add_new_item' => __( 'Add New Rate Table' , 'pmrates' ),
            'edit_item' => __( 'Edit Rate Table' , 'pmrates' ),
            'new_item' => __( 'New Rate Table' , 'pmrates' ),
            'view_item' => __( 'View Rate Table' , 'pmrates' ),
            'search_items' => __( 'Search Rate Tables' , 'pmrates' ),
            'not_found' =>  __( 'No Rate Tables Found' , 'pmrates' ),
            'not_found_in_trash' => __( 'No Rate Tables found in Trash' , 'pmrates' ),
        );

        $args = array(
            'labels' => $labels,
            'menu_icon' => 'dashicons-chart-area',
            'has_archive' => false,
            'public' => true,
            'hierarchical' => false,
            'show_ui' => true,
            'show_in_menu' => true,
            'exclude_from_search' => true,
            'publicly_queryable' => false,
            'supports' => array(
                'title',
                'thumbnail',
                'revisions',
            ),
            'rewrite' => false,
            'show_in_rest' => true
        );
        register_post_type( 'pmrates_rate', $args);
    }
    add_action( 'init', 'pmrates_register_post_type' );


    add_action( 'admin_print_footer_scripts', function () {
        ?>
            <script type="text/javascript">
                document.addEventListener('DOMContentLoaded', function () {
                    if (typeof tinymce !== 'undefined' && document.querySelector(".pmrates-pmrates-controls")) {
                        tinymce.on('SetupEditor', function (editor) {
                            // Could use new 'input' event instead.
                            editor.editor.on("blur",function (event) {
                                pmSaveTable();
                            });
                        });
                    }
                }, false);
            </script>
        <?php
    } );


    /* Meta box setup function. */
    function pmrates_post_meta_boxes_setup() {
        /* Add meta boxes on the 'add_meta_boxes' hook. */
        add_action( 'add_meta_boxes_pmrates_rate', 'pmrates_add_post_meta_boxes' );
        /* Save post meta on the 'save_post' hook. */
        add_action( 'save_post_pmrates_rate', 'pmrates_save_post_meta_boxes', 10, 2 );
    }

    /* Create one or more meta boxes to be displayed on the post editor screen. */
    function pmrates_add_post_meta_boxes($post) {
      add_meta_box(
            'pmrates_rateBox', // Unique ID
            'Rate Table', // Title
            'pmrates_post_call_meta_box', // Callback function
            'pmrates_rate', // Admin page (or post type)
            'normal', // Context
            'high' // Priority
        );
    }

    /** CREATE WIDGET FOR RATE TABLE **/
    include( plugin_dir_path( __FILE__ ) . 'block/pm-rates-block.php');

    /** CODE FOR BUILDING RATE HTML **/
    include( plugin_dir_path( __FILE__ ) . 'includes/pm-rates-builder.php');

    /* Display the post meta box. */
    function pmrates_post_call_meta_box( $post ) { ?>
        <?php wp_nonce_field( basename( __FILE__ ), 'pmrates_nonce' ); ?>
        <div class="pmrates-control-area">

            <p class='table_title_holder'>
                <label for="table_title">Table Title</label>
                <br />
                <input id="table_title" name="table_title" type="text" value="<?php echo $post->table_title; ?>" />
            </p>

            <p class='table_subtitle_holder'>
                <label for="table_subtitle">Table Subtitle (Optional, bold tagline under title.)</label>
                <br />
                <input id="table_subtitle" name="table_subtitle" type="text" value="<?php echo $post->table_subtitle; ?>" />
            </p>


            <div id='table_intro_holder'>
                <label for="table_intro">Table Intro (Appears underneath the Title and Subtitle, but before the Effective Rate Date.)</label>
                <br />
                <?php
                    $settings = array(
                        'wpautop' => false,
                        'media_buttons' => false,
                        'textarea_rows' => 4,
                        'teeny' => true
                    );
                    wp_editor( html_entity_decode($post->table_intro), 'table_intro', $settings );
                ?>
            </div>

            <p class='show_effective_date_holder'>
                <label for="show_effective_date">Show Effective Rate Date</label>
                <br />
                <select id="show_effective_date" name="show_effective_date">
                    <option value="true"<?php if($post->show_effective_date == "true"): ?> selected<?php endif; ?>>Yes</option>
                    <option value="false"<?php if($post->show_effective_date == "false"): ?> selected<?php endif; ?>>No</option>
                </select>
            </p>

            <p class='effective_date_holder'>
                <label for="effective_date">Effective Rate Date (Leaving blank will output the day's date if "Show Effective Rate Date" is set to "Yes.")</label>
                <br />
                <input id="effective_date" name="effective_date" type="date" value="<?php if(isset($post->effective_date) && !empty($post->effective_date)) { echo $post->effective_date; } ?>" />
                <span>Short Code for Effective Date - [pmrateseffectivedate tableid=<?php echo $post->ID; ?>]</span>
            </p>

            <p class='table_caption_holder'>
                <label for="table_caption">Table Caption (Required for ADA)</label>
                <br />
                <input id="table_caption" name="table_caption" type="text" value="<?php echo $post->table_caption; ?>" />
            </p>

            <div id="ratesBuild">
                <?php BuildTable($post->table_data); ?>
            </div>

            <input type="hidden" id="pmRateTable-id" value="<?php echo $post->ID; ?>" />

            <div class="pmRateTable-edit-modal">
                <div class="pmRateTable-modal-content">
                    <textarea id="pmRateTable-edit-ta"></textarea>
                    <div class="pmRateTable-modal-actions">
                        <button class="pmRateTable-modal-update">Update</button>
                        <button class="pmRateTable-modal-cancel">Cancel</button>
                    </div>
                </div>
            </div>

            <div id='table_disclosure_holder'>
                <label for="table_disclosure">Table Disclosure</label>
                <br />
                <?php
                    $settings = array(
                        'wpautop' => false,
                        'media_buttons' => false,
                        'textarea_rows' => 4,
                        'teeny' => true
                    );
                    wp_editor( html_entity_decode($post->table_disclosure), 'table_disclosure', $settings );
                ?>
            </div>

            <ul id="pmrates-cta-container">
                <?php
                    $table_ctas = $post->cta_data;
                    $ctas_json = array();
                    if($table_ctas) {
                        $cta_json = preg_split("/::ctadivide::/", html_entity_decode($table_ctas, ENT_QUOTES));
                        foreach($cta_json as $i => $cta) {
                            $cta_json[$i] = json_decode($cta, true);
                        }
                    }
                ?>
                <?php if($table_ctas): ?>

                    <?php foreach($cta_json as $cta): ?>

                        <li>
                            <p>CTA <span class="pmrates-cta-list-number">1</span> - <span class="pmrates-cta-list-text"></span><button class="pmrates-delete-cta" type="button">&times;</button><button class="pmrates-move-cta-up" type="button">&uarr;</button><button class="pmrates-move-cta-down" type="button">&darr;</button></p>
                            <input type="text" class="pmrates-cta-text" placeholder="CTA Text" onchange="updateListingText(this)" oninput="updateListingText(this)" value="<?php echo $cta['text']; ?>" />
                            <input type="text" class="pmrates-cta-ada-text" placeholder="CTA ADA Text" value="<?php echo $cta['ada_text']; ?>" />
                            <select class="pmrates-cta-url">
                                <option value="">-- Select a Page --</option>
                                <?php
                                    $pages = get_pages();
                                    foreach ( $pages as $page ) {
                                        $option = "";
                                        if(intVal($cta['url']) == intVal($page->ID)) {
                                            $option .= '<option value="' . $page->ID . '" selected>';
                                        } else {
                                            $option .= '<option value="' . $page->ID . '">';
                                        }
                                        $option .= $page->post_title;
                                        $option .= '</option>';
                                        echo $option;
                                    }
                                ?>
                            </select>
                            <?php if(isset($cta['anchor'])): ?>
                                <input type="text" class="pmrates-cta-anchor" placeholder="Optional Anchor for Page Links" value="<?php echo $cta['anchor']; ?>" pattern="#[A-Za-z0-9\-_]+" />
                            <?php else: ?>
                                <input type="text" class="pmrates-cta-anchor" placeholder="Optional Anchor for Page Links" value="" pattern="#[A-Za-z0-9\-_]+" />
                            <?php endif; ?>
                            <input type="text" class="pmrates-cta-custom-url" placeholder="Custom URL (Overrides Page Selection)" value="<?php echo $cta['custom_url']; ?>" />
                            <select class="pmrates-cta-target">
                                <option value="_self"<?php if($cta['target'] == '_self'): ?> selected<?php endif; ?>>Open Link in the Same Tab</option>
                                <option value="_blank"<?php if($cta['target'] == '_blank'): ?> selected<?php endif; ?>>Open Link in the New Tab</option>
                            </select>
                            <input type="text" class="pmrates-cta-custom-class" placeholder="Custom Class" value="<?php if(!empty($cta["custom_class"])) { echo $cta['custom_class']; } ?>" />
                        </li>

                    <?php endforeach; ?>

                <?php else: ?>

                    <li>
                        <p>CTA <span class="pmrates-cta-list-number">1</span> - <span class="pmrates-cta-list-text"></span><button class="pmrates-delete-cta" type="button">&times;</button><button class="pmrates-move-cta-up" type="button">&uarr;</button><button class="pmrates-move-cta-down" type="button">&darr;</button></p>
                        <input type="text" class="pmrates-cta-text" placeholder="CTA Text" onchange="updateListingText(this)" oninput="updateListingText(this)" value="" />
                        <input type="text" class="pmrates-cta-ada-text" placeholder="CTA ADA Text" value="" />
                        <select class="pmrates-cta-url">
                            <option value="" selected>-- Select a Page --</option>
                            <?php
                                $pages = get_pages();
                                foreach ( $pages as $page ) {
                                    $option = '<option value="' . $page->ID . '">';
                                    $option .= $page->post_title;
                                    $option .= '</option>';
                                    echo $option;
                                }
                            ?>
                        </select>
                        <input type="text" class="pmrates-cta-anchor" placeholder="Optional Anchor for Page Links" value="" pattern="#[A-Za-z0-9\-_]+" />
                        <input type="text" class="pmrates-cta-custom-url" placeholder="Custom URL (Overrides Page Selection)" />
                        <select class="pmrates-cta-target">
                            <option value="_self" selected>Open Link in the Same Tab</option>
                            <option value="_blank">Open Link in the New Tab</option>
                        </select>
                        <input type="text" class="pmrates-cta-custom-class" placeholder="Custom Class" />
                    </li>

                <?php endif; ?>
            </ul>

            <div class="pmrates-cta-controls">
                <button onclick="pmAddCTA(); return false;" type="button">Add CTA</button>
            </div>
            <div style="display: none;" id="pmrates-page-list-json">
            <?php
                $pages = get_pages();
                $build = "";
                foreach($pages as $page) {
                    $build .= '[{"id":'.$page->ID.',"title":"'.htmlentities($page->post_title, ENT_QUOTES).'"}]::pagedivide::';
                }
                echo $build;
            ?>
            </div>

            <p style="display: none;">
                <label for="table_data">This field contains the table data converted to JSON by cell.</label>
                <br />
                <input type="hidden" name="table_data" id="table_data" value="<?php echo esc_attr( $post->table_data ); ?>" size="30" />
            </p>

            <p style="display: none;">
                <label for="cta_data">This field contains the cta data converted to JSON by cta.</label>
                <br />
                <input type="hidden" name="cta_data" id="cta_data" value="<?php echo esc_attr( $post->cta_data ); ?>" size="30" />
            </p>

        </div>
    <?php }


    add_action( 'load-post.php', 'pmrates_post_meta_boxes_setup' );
    add_action( 'load-post-new.php', 'pmrates_post_meta_boxes_setup' );

    function update_pmrates_post_meta($post_id, $meta_key, $new_meta_value) {
        $meta_value = get_post_meta($post_id, $meta_key, true);
        if ($new_meta_value && '' === $meta_value) {
            add_post_meta($post_id, $meta_key, $new_meta_value, true); // unique
        } else if ($new_meta_value && $new_meta_value !== $meta_value) {
            update_post_meta($post_id, $meta_key, $new_meta_value, $meta_value);
            // same prev_value
        } else if ('' === $new_meta_value && $meta_value) {
            delete_post_meta($post_id, $meta_key, $meta_value);
        }
    }

    /* Save the meta box’s post metadata. */
    function pmrates_save_post_meta_boxes( $post_id, $post ) {

        /* Verify the nonce before proceeding. */
        if ( !isset( $_POST['pmrates_nonce'] ) || !wp_verify_nonce( $_POST['pmrates_nonce'], basename( __FILE__ ) ) ) {
            return $post_id;
        }

        /* Get the post type object. */
        $post_type = get_post_type_object( $post->post_type );

        /* Check if the current user has permission to edit the post. */
        if ( !current_user_can( $post_type->cap->edit_post, $post_id ) ) {
            return $post_id;
        }

        foreach ($_POST as $key => $value) {
            update_pmrates_post_meta($post_id, $key, $value);
        }
    }


    /*** SHORT CODES ***/
    function pmrates_display_cell( $atts = array() ) {
        // set up default parameters
        extract(shortcode_atts(array(
            'tableid' => '',
            'rowid' => '',
            'colid' => ''
        ), $atts));

        $tableID = $atts['tableid'] ?? '';
        $rate = get_post_meta( $tableID, 'table_data', true );
        $tableCellValue = getRateTableCellData($rate, $atts["rowid"], $atts["cellid"]);

        return $tableCellValue;
    }
    add_shortcode('pmrates', 'pmrates_display_cell');

    function pmrates_display_disclosure( $atts = array() ) {
        extract (shortcode_atts(array(
            'tableid' => ''
        ), $atts));

        $tableID = $atts['tableid'] ?? '';
        $table_disclosure = get_post_meta($tableID, 'table_disclosure', true);
        return $table_disclosure;
    }
    add_shortcode('pmratesdisclosure', 'pmrates_display_disclosure');

    function pmrates_display_effective_date( $atts = array() ) {
        extract (shortcode_atts(array(
            'tableid' => ''
        ), $atts));

        $tableID = $atts['tableid'] ?? '';
        $formattedDate = date("n.j.Y");
        if(!empty($tableID)) {
            $table_effective_date = get_post_meta($tableID, 'effective_date', true);
            if(isset($table_effective_date) && !empty($table_effective_date)) {
                $formattedDate = date("n.j.Y", strtotime($table_effective_date));
            }
        }

        return $formattedDate;
    }
    add_shortcode('pmrateseffectivedate', 'pmrates_display_effective_date');

    // Register text used in the plugin for translation
    function pmrates_register_translation_strings() {
        if(function_exists('pll_register_string')) {
            pll_register_string( 'pm_rates_accurate-as-of', 'Rates accurate as of', 'Pannos Rate Table Plugin', false );
            pll_register_string( 'pm_rates_subject-to-change', 'All rates and terms subject to change without notice.', 'Pannos Rate Table Plugin', false);
        }
    }
    add_action( 'init', 'pmrates_register_translation_strings' );



add_action( "rest_insert_pmrates_rate", function ( \WP_Post $post, $request, $creating ) 
{
    $parameters = $request->get_query_params();    
    $metas = $parameters['meta'];
    error_log("Rate Table Inserted. ".print_r($post,true));
    if( is_array( $metas ) ) {
        foreach( $metas as $name => $value ) {

            if($name == 'table_data'){
                $table_data_rows = $value;
                $starting_row = 2000;
                $starting_cell= 3000;
                $cellheaders = array();
                $json = array();
                foreach ($table_data_rows as $RowNumeric => $rowCells) {
                    $RowID = $starting_row + $RowNumeric;
                    foreach ($rowCells as $ColNumeric => $CellValue) {
                        $json_cell = '';
                        if($RowNumeric == 0){
                            $cellheaders[$ColNumeric] = rate_cel_encode(preg_replace('#[^\w()/.%\&]#',"_",strtolower($CellValue)).'_'.$post->ID);
                        }
                        $ColID = $starting_cell + $ColNumeric;
                        $json_cell .= '{';
                        $json_cell .= '"RowID": "'.$RowID.'",';
                        $json_cell .= '"RowNumeric": "'.$RowNumeric.'",';
                        $json_cell .= '"ColID": "'.$ColID.'",';
                        $json_cell .= '"ColNumeric": "'.$ColNumeric.'",';
                        $json_cell .= '"CellValue": "'.rate_cel_encode(trim(str_replace('"','::quote::',$CellValue))).'",';
                        $json_cell .= '"CellHeaders": "'.$cellheaders[$ColNumeric].'"';
                        $json_cell .= '}';

                        $json[$RowNumeric.'_'.$ColNumeric] = $json_cell;
                    }
                }

                error_log("Rate Table Data: ".print_r($json,true));

                update_pmrates_post_meta($post->ID, 'table_data', implode('::celldivide::',$json));
                error_log("Rate Table Data Encoded: ".implode('::celldivide::',$json));

            } else {
                update_pmrates_post_meta($post->ID, $name, $value);
            }
        }
    }
}, 10, 3 );

function rate_cel_encode($str) {
    $str = mb_convert_encoding($str , 'UTF-32', 'UTF-8');
    $t = unpack("N*", $str);
    $t = array_map(function($n) { return "&#$n;"; }, $t);
    return implode("", $t);
}

?>
